<?php

namespace Drupal\commerce_dias_redirect\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\commerce_payment\Entity\PaymentGateway;

/**
 * Endpoints for the routes defined.
 */
class CallbackController extends ControllerBase {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The DIAS API service.
   *
   * @var \Drupal\commerce_dias_redirect\DiasApiService
   */
  protected $diasApiService;

  /**
   * Constructs a CallbackController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\commerce_dias_redirect\DiasApiService $diasApiService
   *    The DIAS API service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, LoggerChannelFactoryInterface $logger_factory, TimeInterface $time, \Drupal\commerce_dias_redirect\DiasApiService $diasApiService) {
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_factory->get('commerce_dias_redirect');
    $this->time = $time;
    $this->diasApiService = $diasApiService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
      $container->get('datetime.time'),
      $container->get('commerce_dias_redirect.dias_api_service')
    );
  }

  /**
   * Callback action.
   *
   * Listen for callbacks from DIAS environment and creates any payment
   * specified.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\Response|array
   *   The response.
   */
  public function callback(Request $request) {
    $dias_order_id = $request->query->get('orderId');
    $commerce_order_id = $request->attributes->get('commerce_order');

    /** @var PaymentGateway $payment_gateway */
    $payment_gateway = $this->entityTypeManager->getStorage('commerce_payment_gateway')
      ->load('dias_payment');
    $api_order_registration_url = $payment_gateway->getPluginConfiguration()['api_get_order_status_url'];
    $username = $payment_gateway->getPluginConfiguration()['username'];
    $password = $payment_gateway->getPluginConfiguration()['password'];

    // Prepare the request data for the DIAS API service.
    $submission_data = [
      'ApiUrl' => $api_order_registration_url,
      'username' => $username,
      'password' => $password,
      'orderId' => $dias_order_id,
      'language' => $this->languageManager()->getDefaultLanguage()->getId(),
    ];

    $failure_url = Url::fromRoute('commerce.cart');
    // DIAS API service to retrieve the status of the registered order.
    $response = $this->diasApiService->getOrderStatus($submission_data);
    if ($response->getStatusCode() != 200) {
      $this->logger->error('Failed to retrieve the payment URL from DIAS.');
      $this->moduleHandler()
        ->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
      return new RedirectResponse($failure_url->toString());
    }
    $data = json_decode($response->getContent());

    if ($data->status_code != 200) {
      /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
      $order = $this->entityTypeManager->getStorage('commerce_order')
        ->load($commerce_order_id);
      $this->moduleHandler()
        ->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
      return new RedirectResponse($failure_url->toString());
    }

    if (!$commerce_order_id) {
      $this->moduleHandler()
        ->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
      return new RedirectResponse($failure_url->toString());
    }

    $error_code = $data->data->errorCode;
    $order_status = $data->data->orderStatus;
    $action_code = $data->data->actionCode;
    if ($error_code != 0 || $order_status != 2 || $action_code != 0) {
      $this->logResponse($commerce_order_id, $data, "failure");
      $this->messenger()->addError($data->error);
      /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
      $order = $this->entityTypeManager->getStorage('commerce_order')
        ->load($commerce_order_id);
      $this->moduleHandler()
        ->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
      return new RedirectResponse($failure_url->toString());
    }

    $this->createPayment($request, $data);
    $this->logResponse($commerce_order_id, $data, "success");
    return new RedirectResponse(Url::fromRoute('commerce_checkout.form', [
      'commerce_order' => $commerce_order_id,
      'step_id' => 'complete',
    ])->toString());
  }

  /**
   * Log response data to database log and order object.
   */
  public function logResponse($commerce_order_id, $data, $status = "success") {
    // Log for immediate action.
    if ($status === "success") {
      $this->logger->info('Order ' . $commerce_order_id . ' payment ' . $status . ':  ' . $data->data->actionCodeDescription);
    }
    elseif ($status === "failure") {
      $this->logger->error('Order ' . $commerce_order_id . ' payment ' . $status . ':  ' . $data->data->actionCodeDescription);
    }

    // Log in the order data field for future reference.
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $this->entityTypeManager->getStorage('commerce_order')
      ->load($commerce_order_id);
    // Extract and validate necessary fields from the incoming data.
    $payment_date_time = $data->data->authDateTime ?? NULL;
    $approval_code = $data->data->cardAuthInfo->approvalCode ?? NULL;
    $diasGatewayData = $order->getData("DiasGatewayData");
    $diasGatewayData = array_merge($diasGatewayData, [
      "status" => $data->data->orderStatus,
      "orderNumber" => $data->data->orderNumber,
      "rrn" => $data->data->authRefNum,
      "paymentDateTime" => $payment_date_time,
      "cardPan" => $data->data->cardAuthInfo->pan,
      "approvalCode" => $approval_code,
      "data" => $data->data,
    ]);
    $order->setData("DiasGatewayData", $diasGatewayData);
    $order->save();
  }

  /**
   * Create Payment.
   */
  public function createPayment($request, $data, $state = "completed") {
    $dias_order_id = $request->query->get('orderId');
    $commerce_order_id = $request->attributes->get('commerce_order');
    $order = $this->entityTypeManager->getStorage('commerce_order')
      ->load($commerce_order_id);

    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    $gateway_data = $order->getData("DiasGatewayData");
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $payment_storage->create([
      'state' => 'completed',
      'amount' => $order->getBalance(),
      'payment_gateway' => $gateway_data["paymentGateway"],
      'order_id' => $order->id(),
      'remote_state' => $data->data->orderStatus,
      'message' => $data->data->actionCodeDescription,
      'remote_id' => $dias_order_id,
    ]);
    if ($state === 'completed') {
      $payment->setAuthorizedTime(\Drupal::time()->getRequestTime());
      $payment->setCompletedTime($this->time->getRequestTime());
    }
    $payment->save();
  }

}
