<?php

namespace Drupal\commerce_dias_redirect;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Url;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class DiasApiService.
 *
 * @package Drupal\amco_api_integration
 */
class DiasApiService {

  /**
   * The HTTP client.
   *
   * @var ClientInterface
   */
  protected ClientInterface $httpClient;

  /**
   * The Config Factory
   *
   * @var ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a new DiasApiService object.
   *
   */
  public function __construct(ClientInterface $httpClient, ConfigFactoryInterface $configFactory, LoggerChannelFactoryInterface $logger_factory) {
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->logger = $logger_factory->get('commerce_dias_redirect');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('logger.factory'),
    );
  }

  /**
   * Creates a JsonResponse with the provided data, error message, and status
   * code.
   *
   * @param mixed $data
   *   The data to be included in the response, serializable to JSON.
   * @param string $error
   *   An optional error message; use an empty string if none is needed.
   * @param int $statusCode
   *   A valid HTTP status code (e.g., 200, 400, 500).
   *
   * @return JsonResponse
   *   A JsonResponse object containing the specified data, error, and status
   *   code.
   */
  protected function createJsonResponse(mixed $data, string $error, int $statusCode) {
    return new JsonResponse([
      'data' => $data,
      'error' => $error,
      'status_code' => $statusCode,
    ]);
  }

  /**
   * Register an order to DIAS Payment gateway.
   *
   * @param array $submission_data
   *    The submission data.
   *
   */
  public function orderRegistration(array $submission_data) {
    $api_url = $submission_data['ApiUrl'];
    $api_username = $submission_data['username'];
    $api_password = $submission_data['password'];
    $order_number = $submission_data['orderNumber'];
    $amount = $submission_data['amount'];
    $language = $submission_data['language'];
    $currency = $submission_data['currency'];
    $return_url = $submission_data['returnUrl'];
    $full_url = Url::fromUri($api_url . '?userName=' . $api_username . '&password=' . $api_password . '&orderNumber=' . $order_number . '&amount=' . $amount . '&currency=' . $currency . '&returnUrl=' . $return_url . '&language=' . $language)
      ->toString();

    // Attempt to send the data to the API.
    try {
      $response = $this->httpClient->post($full_url, [
        'json' => $submission_data,
        'headers' => [
          'Content-Type' => 'application/json',
        ],
      ]);
      $data = (string) $response->getBody();
      $data = json_decode($data);
      if (isset($data->data->errorCode)) {
        $error_code = $data->data->errorCode;
        if ($error_code > 0) {
          $this->logger->error('Request failed with error code @error_code: @error_code_message', [
            '@error_code' => $data->errorCode,
            '@error_code_message' => $data->errorMessage,
          ]);
          $error_message = t('Transaction rejected. Please contact with your bank.');
          return $this->createJsonResponse($data, $error_message, 200);
        }
      }
      $this->logger->info('Request to @endpoint was successful.', ['@endpoint' => $api_url]);
      return $this->createJsonResponse($data, '', 200);
    }
    catch (\Exception $e) {
      $errorMessage = $e->getMessage();
      $statusCode = isset($response) ? $response->getStatusCode() : 500;
      $this->logger->error('An error occurred: @message', ['@message' => $errorMessage]);
      return $this->createJsonResponse('', $errorMessage, $statusCode);
    }
  }

  /**
   * Get information about the status of a DIAS Payment gateway order.
   *
   * @param array $submission_data
   *    The submission data.
   *
   */
  public function getOrderStatus(array $submission_data) {
    $api_url = $submission_data['ApiUrl'];
    $api_username = $submission_data['username'];
    $api_password = $submission_data['password'];
    $order_id = $submission_data['orderId'];
    $language = $submission_data['language'];
    $full_url = Url::fromUri($api_url . '?userName=' . $api_username . '&password=' . $api_password . '&orderId=' . $order_id . '&language=' . $language)
      ->toString();

    // Attempt to send the data to the API.
    try {
      $response = $this->httpClient->post($full_url, [
        'json' => $submission_data,
        'headers' => [
          'Content-Type' => 'application/json',
        ],
      ]);
      $data = (string) $response->getBody();
      $data = json_decode($data);
      if ($data->errorCode != 0) {
        $this->logger->error('Request failed with error code @error_code: @error_code_message', [
          '@error_code' => $data->errorCode,
          '@error_code_message' => $data->errorMessage,
        ]);
        $error_message = t('Transaction rejected. Please contact with your bank.');
        return $this->createJsonResponse($data, $error_message, 200);
      }
      if ($data->orderStatus != 2) {
        switch ($data->orderStatus) {
          case 0:
            $order_message = 'Order is registered. No payment attempts have been made yet.';
            break;
          case 5:
            $order_message = 'Authentication in progress.';
            break;
          case 6:
            $order_message = 'Transaction is rejected (Final status).';
            break;
        }
        $this->logger->error('Request failed with order status @order_status: @order_status_message', [
          '@order_status' => $data->orderStatus,
          '@order_status_message' => !empty($order_message) ? $order_message : 'System error',
        ]);
        if ($data->actionCode != 0) {
          $this->logger->error('Request failed with action code @action_code: @action_code_message', [
            '@action_code' => $data->actionCode,
            '@action_code_message' => $data->actionCodeDescription,
          ]);
          $error_message = t('Transaction rejected. Please contact with your bank.');
          if ($data->actionCode == -2013) {
            $error_message = t('You canceled your transaction.');
          }
          return $this->createJsonResponse($data, $error_message, 200);
        }
      }
      $this->logger->info('Request to @endpoint was successful.', ['@endpoint' => $api_url]);
      return $this->createJsonResponse($data, '', 200);
    }
    catch (\Exception $e) {
      $error_message = $e->getMessage();
      $status_code = isset($response) ? $response->getStatusCode() : 500;
      $this->logger->error('An error occurred: @message', ['@message' => $error_message]);
      return $this->createJsonResponse('', $error_message, $status_code);
    }
  }

}
