<?php

namespace Drupal\commerce_dias_redirect\PluginForm\OffsiteRedirect;

use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_payment\Exception\InvalidResponseException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Provides the DIAS payment redirect form.
 */
class DiasPaymentRedirectForm extends BasePaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The DIAS API service.
   *
   * @var \Drupal\commerce_dias_redirect\DiasApiService
   */
  protected $diasApiService;

  /**
   * Constructs a new DiasPaymentRedirectForm object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\commerce_dias_redirect\DiasApiService $diasApiService
   *   The DIAS API service.
   */
  public function __construct(LoggerChannelFactoryInterface $logger_factory, \Drupal\commerce_dias_redirect\DiasApiService $diasApiService) {
    $this->logger = $logger_factory->get('commerce_dias_redirect');
    $this->diasApiService = $diasApiService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory'),
      $container->get('commerce_dias_redirect.dias_api_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // Get the payment and order data.
    $payment = $this->entity;
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $payment->getOrder();
    $config = $payment->getPaymentGateway()->getPlugin()->getConfiguration();

    // The amount in minimal currency unit (cents).
    //    $amount = sprintf('%0.2f', $order->getTotalPrice()->getNumber());
    $amount = (int) $order->getTotalPrice()->getNumber() * 100;

    // Validate the necessary configuration values.
    if (empty($config['api_order_registration_url']) || empty($config['username']) || empty($config['password'])) {
      $this->logger->error('DIAS API credentials are not set properly. Please configure them in the settings.');
      return [];
    }
    $current_timestamp = time();

    // Prepare the request data for the DIAS API service.
    $submission_data = [
      'ApiUrl' => $config['api_order_registration_url'],
      'username' => $config['username'],
      'password' => $config['password'],
      'orderNumber' => $order->id() . '-' . $current_timestamp,
      'amount' => $amount,
      'currency' => '978',
      'language' => \Drupal::languageManager()->getDefaultLanguage()->getId(),
      'returnUrl' => \Drupal\Core\Url::fromRoute('commerce_dias_redirect.payment_callback', ['commerce_order' => $order->id()], ['absolute' => TRUE])
        ->toString(),
    ];

    $failure_url = Url::fromRoute('commerce.cart');
    // Call the DIAS API service to register the order and get the payment URL.
    try {
      $response = $this->diasApiService->orderRegistration($submission_data);
      $data = json_decode($response->getContent());
      if ($response->getStatusCode() != 200) {
        \Drupal::messenger()->addError($data->error);
        \Drupal::moduleHandler()->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
        return new RedirectResponse($failure_url->toString());
      }

      if (isset($data->data->errorCode)) {
        $error_code = $data->data->errorCode;
        if ($error_code > 0) {
          \Drupal::messenger()->addError($data->error);
          \Drupal::moduleHandler()
            ->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
          return new RedirectResponse($failure_url->toString());
        }
      }

      // Extract the payment URL from the response.
      $paymentUrl = $data->data->formUrl ?? NULL;

      // Ensure the payment URL was returned.
      if (!$paymentUrl) {
        $this->logger->error('No payment URL was received from DIAS.');
        \Drupal::moduleHandler()->alter('commerce_dias_redirect_failure_url', $failure_url, $order);
        return new RedirectResponse($failure_url->toString());
      }

      // Save response data for validating response later.
      $order->setData("DiasGatewayData", [
        'orderId' => $data->data->orderId,
        'paymentGateway' => $payment->getPaymentGateway()->id(),
      ]);
      $order->save();

      // Prepare the redirect form data.
      $form_data = [
        'PaymentUrl' => $paymentUrl,
      ];

      // Build the form for redirection.
      $form = $this->buildRedirectForm($form, $form_state, $paymentUrl, $form_data, 'post');

    }
    catch (InvalidResponseException $e) {
      $this->logger->error('Error during payment URL creation: @message', ['@message' => $e->getMessage()]);
      $form['#markup'] = $this->t('There was an error while processing your payment. Please try again later.');
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // No submission action needed because the user will be redirected to DIAS.
  }

}
